@@ -37,6 +37,8 @@
 #else
 #include <CL/cl.h>
 #endif 
+#elif(VKFFT_BACKEND==4)
+#include <ze_api.h>
 #endif
 #include "vkFFT.h"
 #include "half.hpp"
@@ -60,6 +62,8 @@ VkFFTResult sample_13_precision_VkFFT_half(VkGPU* vkGPU, uint64_t file_output, F
 	hipError_t res = hipSuccess;
 #elif(VKFFT_BACKEND==3)
 	cl_int res = CL_SUCCESS;
+#elif(VKFFT_BACKEND==4)
+	ze_result_t res = ZE_RESULT_SUCCESS;
 #endif
 	if (file_output)
 		fprintf(output, "13 - VkFFT/FFTW C2C precision test in half precision\n");
@@ -130,7 +134,7 @@ VkFFTResult sample_13_precision_VkFFT_half(VkGPU* vkGPU, uint64_t file_output, F
 #ifdef USE_cuFFT
 			half2* output_extFFT = (half2*)(malloc(2 * sizeof(half) * dims[0] * dims[1] * dims[2]));
 			if (!output_extFFT) return VKFFT_ERROR_MALLOC_FAILED;
-			launch_precision_cuFFT_half(inputC, (void*)output_extFFT, benchmark_dimensions[n]);
+			launch_precision_cuFFT_half(inputC, (void*)output_extFFT, (int)vkGPU->device_id, benchmark_dimensions[n]);
 #endif // USE_cuFFT
 
 			//VkFFT part
@@ -152,8 +156,11 @@ VkFFTResult sample_13_precision_VkFFT_half(VkGPU* vkGPU, uint64_t file_output, F
 			configuration.physicalDevice = &vkGPU->physicalDevice;
 			configuration.isCompilerInitialized = isCompilerInitialized;//compiler can be initialized before VkFFT plan creation. if not, VkFFT will create and destroy one after initialization
 #elif(VKFFT_BACKEND==3)
-			configuration.platform = &vkGPU->platform;
 			configuration.context = &vkGPU->context;
+#elif(VKFFT_BACKEND==4)
+			configuration.context = &vkGPU->context;
+			configuration.commandQueue = &vkGPU->commandQueue;
+			configuration.commandQueueID = vkGPU->commandQueueID;
 #endif
 
 			uint64_t numBuf = 1;
@@ -176,6 +183,8 @@ VkFFTResult sample_13_precision_VkFFT_half(VkGPU* vkGPU, uint64_t file_output, F
 			hipFloatComplex* buffer = 0;
 #elif(VKFFT_BACKEND==3)
 			cl_mem buffer = 0;
+#elif(VKFFT_BACKEND==4)
+			void* buffer = 0;
 #endif			
 			for (uint64_t i = 0; i < numBuf; i++) {
 #if(VKFFT_BACKEND==0)
@@ -192,6 +201,11 @@ VkFFTResult sample_13_precision_VkFFT_half(VkGPU* vkGPU, uint64_t file_output, F
 #elif(VKFFT_BACKEND==3)
 				buffer = clCreateBuffer(vkGPU->context, CL_MEM_READ_WRITE, bufferSize[i], 0, &res);
 				if (res != CL_SUCCESS) return VKFFT_ERROR_FAILED_TO_ALLOCATE;
+#elif(VKFFT_BACKEND==4)
+				ze_device_mem_alloc_desc_t device_desc = {};
+				device_desc.stype = ZE_STRUCTURE_TYPE_DEVICE_MEM_ALLOC_DESC;
+				res = zeMemAllocDevice(vkGPU->context, &device_desc, bufferSize[i], sizeof(float), vkGPU->device, &buffer);
+				if (res != ZE_RESULT_SUCCESS) return VKFFT_ERROR_FAILED_TO_ALLOCATE;
 #endif
 			}
 
@@ -222,6 +236,23 @@ VkFFTResult sample_13_precision_VkFFT_half(VkGPU* vkGPU, uint64_t file_output, F
 #elif(VKFFT_BACKEND==3)
 				res = clEnqueueWriteBuffer(vkGPU->commandQueue, buffer, CL_TRUE, 0, bufferSize[i], inputC, 0, NULL, NULL);
 				if (res != CL_SUCCESS) return VKFFT_ERROR_FAILED_TO_COPY;
+#elif(VKFFT_BACKEND==4)
+				ze_command_queue_desc_t commandQueueCopyDesc = {
+					ZE_STRUCTURE_TYPE_COMMAND_QUEUE_DESC,
+					0,
+					vkGPU->commandQueueID,
+					0, // index
+					0, // flags
+					ZE_COMMAND_QUEUE_MODE_DEFAULT,
+					ZE_COMMAND_QUEUE_PRIORITY_NORMAL
+				};
+				ze_command_list_handle_t copyCommandList;
+				res = zeCommandListCreateImmediate(vkGPU->context, vkGPU->device, &commandQueueCopyDesc, &copyCommandList);
+				if (res != ZE_RESULT_SUCCESS) return VKFFT_ERROR_FAILED_TO_CREATE_COMMAND_LIST;
+				res = zeCommandListAppendMemoryCopy(copyCommandList, buffer, inputC, bufferSize[i], 0, 0, 0);
+				if (res != ZE_RESULT_SUCCESS) return VKFFT_ERROR_FAILED_TO_COPY;
+				res = zeCommandQueueSynchronize(vkGPU->commandQueue, UINT32_MAX);
+				if (res != 0) return VKFFT_ERROR_FAILED_TO_SYNCHRONIZE;
 #endif
 				shift += bufferSize[i];
 			}
@@ -240,6 +271,8 @@ VkFFTResult sample_13_precision_VkFFT_half(VkGPU* vkGPU, uint64_t file_output, F
 			launchParams.buffer = (void**)&buffer;
 #elif(VKFFT_BACKEND==3)
 			launchParams.buffer = &buffer;
+#elif(VKFFT_BACKEND==4)
+			launchParams.buffer = (void**)&buffer;
 #endif
 			resFFT = performVulkanFFT(vkGPU, &app, &launchParams, -1, num_iter);
 			if (resFFT != VKFFT_SUCCESS) return resFFT;
@@ -260,6 +293,23 @@ VkFFTResult sample_13_precision_VkFFT_half(VkGPU* vkGPU, uint64_t file_output, F
 #elif(VKFFT_BACKEND==3)
 				res = clEnqueueReadBuffer(vkGPU->commandQueue, buffer, CL_TRUE, 0, bufferSize[i], output_VkFFT, 0, NULL, NULL);
 				if (res != CL_SUCCESS) return VKFFT_ERROR_FAILED_TO_COPY;
+#elif(VKFFT_BACKEND==4)
+				ze_command_queue_desc_t commandQueueCopyDesc = {
+					ZE_STRUCTURE_TYPE_COMMAND_QUEUE_DESC,
+					0,
+					vkGPU->commandQueueID,
+					0, // index
+					0, // flags
+					ZE_COMMAND_QUEUE_MODE_DEFAULT,
+					ZE_COMMAND_QUEUE_PRIORITY_NORMAL
+				};
+				ze_command_list_handle_t copyCommandList;
+				res = zeCommandListCreateImmediate(vkGPU->context, vkGPU->device, &commandQueueCopyDesc, &copyCommandList);
+				if (res != ZE_RESULT_SUCCESS) return VKFFT_ERROR_FAILED_TO_CREATE_COMMAND_LIST;
+				res = zeCommandListAppendMemoryCopy(copyCommandList, output_VkFFT, buffer, bufferSize[i], 0, 0, 0);
+				if (res != ZE_RESULT_SUCCESS) return VKFFT_ERROR_FAILED_TO_COPY;
+				res = zeCommandQueueSynchronize(vkGPU->commandQueue, UINT32_MAX);
+				if (res != 0) return VKFFT_ERROR_FAILED_TO_SYNCHRONIZE;
 #endif
 				shift += bufferSize[i];
 			}
@@ -325,6 +375,8 @@ VkFFTResult sample_13_precision_VkFFT_half(VkGPU* vkGPU, uint64_t file_output, F
 				hipFree(buffer);
 #elif(VKFFT_BACKEND==3)
 				clReleaseMemObject(buffer);
+#elif(VKFFT_BACKEND==4)
+				zeMemFree(vkGPU->context, buffer);
 #endif
 
 			}
