@@ -37,6 +37,8 @@
 #else
 #include <CL/cl.h>
 #endif 
+#elif(VKFFT_BACKEND==4)
+#include <ze_api.h>
 #endif
 #include "vkFFT.h"
 #include "utils_VkFFT.h"
@@ -58,6 +60,8 @@ VkFFTResult sample_14_precision_VkFFT_single_nonPow2(VkGPU* vkGPU, uint64_t file
 	hipError_t res = hipSuccess;
 #elif(VKFFT_BACKEND==3)
 	cl_int res = CL_SUCCESS;
+#elif(VKFFT_BACKEND==4)
+	ze_result_t res = ZE_RESULT_SUCCESS;
 #endif
 	if (file_output)
 		fprintf(output, "14 - VkFFT/FFTW C2C radix 3/5/7/11/13/Bluestein precision test in single precision\n");
@@ -138,12 +142,12 @@ VkFFTResult sample_14_precision_VkFFT_single_nonPow2(VkGPU* vkGPU, uint64_t file
 #ifdef USE_cuFFT
 			fftwf_complex* output_extFFT = (fftwf_complex*)(malloc(sizeof(fftwf_complex) * dims[0] * dims[1] * dims[2]));
 			if (!output_extFFT) return VKFFT_ERROR_MALLOC_FAILED;
-			launch_precision_cuFFT_single(inputC, (void*)output_extFFT, benchmark_dimensions[n]);
+			launch_precision_cuFFT_single(inputC, (void*)output_extFFT, (int)vkGPU->device_id, benchmark_dimensions[n]);
 #endif // USE_cuFFT
 #ifdef USE_rocFFT
 			fftwf_complex* output_extFFT = (fftwf_complex*)(malloc(sizeof(fftwf_complex) * dims[0] * dims[1] * dims[2]));
 			if (!output_extFFT) return VKFFT_ERROR_MALLOC_FAILED;
-			launch_precision_rocFFT_single(inputC, (void*)output_extFFT, benchmark_dimensions[n]);
+			launch_precision_rocFFT_single(inputC, (void*)output_extFFT, (int)vkGPU->device_id, benchmark_dimensions[n]);
 #endif // USE_rocFFT
 			//VkFFT part
 
@@ -164,8 +168,11 @@ VkFFTResult sample_14_precision_VkFFT_single_nonPow2(VkGPU* vkGPU, uint64_t file
 			configuration.physicalDevice = &vkGPU->physicalDevice;
 			configuration.isCompilerInitialized = isCompilerInitialized;//compiler can be initialized before VkFFT plan creation. if not, VkFFT will create and destroy one after initialization
 #elif(VKFFT_BACKEND==3)
-			configuration.platform = &vkGPU->platform;
 			configuration.context = &vkGPU->context;
+#elif(VKFFT_BACKEND==4)
+			configuration.context = &vkGPU->context;
+			configuration.commandQueue = &vkGPU->commandQueue;
+			configuration.commandQueueID = vkGPU->commandQueueID;
 #endif			
 
 			uint64_t numBuf = 1;
@@ -188,6 +195,8 @@ VkFFTResult sample_14_precision_VkFFT_single_nonPow2(VkGPU* vkGPU, uint64_t file
 			hipFloatComplex* buffer = 0;
 #elif(VKFFT_BACKEND==3)
 			cl_mem buffer = 0;
+#elif(VKFFT_BACKEND==4)
+			void* buffer = 0;
 #endif			
 			for (uint64_t i = 0; i < numBuf; i++) {
 #if(VKFFT_BACKEND==0)
@@ -204,6 +213,11 @@ VkFFTResult sample_14_precision_VkFFT_single_nonPow2(VkGPU* vkGPU, uint64_t file
 #elif(VKFFT_BACKEND==3)
 				buffer = clCreateBuffer(vkGPU->context, CL_MEM_READ_WRITE, bufferSize[i], 0, &res);
 				if (res != CL_SUCCESS) return VKFFT_ERROR_FAILED_TO_ALLOCATE;
+#elif(VKFFT_BACKEND==4)
+				ze_device_mem_alloc_desc_t device_desc = {};
+				device_desc.stype = ZE_STRUCTURE_TYPE_DEVICE_MEM_ALLOC_DESC;
+				res = zeMemAllocDevice(vkGPU->context, &device_desc, bufferSize[i], sizeof(float), vkGPU->device, &buffer);
+				if (res != ZE_RESULT_SUCCESS) return VKFFT_ERROR_FAILED_TO_ALLOCATE;
 #endif
 			}
 
@@ -234,6 +248,23 @@ VkFFTResult sample_14_precision_VkFFT_single_nonPow2(VkGPU* vkGPU, uint64_t file
 #elif(VKFFT_BACKEND==3)
 				res = clEnqueueWriteBuffer(vkGPU->commandQueue, buffer, CL_TRUE, 0, bufferSize[i], inputC, 0, NULL, NULL);
 				if (res != CL_SUCCESS) return VKFFT_ERROR_FAILED_TO_COPY;
+#elif(VKFFT_BACKEND==4)
+				ze_command_queue_desc_t commandQueueCopyDesc = {
+					ZE_STRUCTURE_TYPE_COMMAND_QUEUE_DESC,
+					0,
+					vkGPU->commandQueueID,
+					0, // index
+					0, // flags
+					ZE_COMMAND_QUEUE_MODE_DEFAULT,
+					ZE_COMMAND_QUEUE_PRIORITY_NORMAL
+				};
+				ze_command_list_handle_t copyCommandList;
+				res = zeCommandListCreateImmediate(vkGPU->context, vkGPU->device, &commandQueueCopyDesc, &copyCommandList);
+				if (res != ZE_RESULT_SUCCESS) return VKFFT_ERROR_FAILED_TO_CREATE_COMMAND_LIST;
+				res = zeCommandListAppendMemoryCopy(copyCommandList, buffer, inputC, bufferSize[i], 0, 0, 0);
+				if (res != ZE_RESULT_SUCCESS) return VKFFT_ERROR_FAILED_TO_COPY;
+				res = zeCommandQueueSynchronize(vkGPU->commandQueue, UINT32_MAX);
+				if (res != 0) return VKFFT_ERROR_FAILED_TO_SYNCHRONIZE;
 #endif
 				shift += bufferSize[i];
 			}
@@ -252,6 +283,8 @@ VkFFTResult sample_14_precision_VkFFT_single_nonPow2(VkGPU* vkGPU, uint64_t file
 			launchParams.buffer = (void**)&buffer;
 #elif(VKFFT_BACKEND==3)
 			launchParams.buffer = &buffer;
+#elif(VKFFT_BACKEND==4)
+			launchParams.buffer = (void**)&buffer;
 #endif
 			resFFT = performVulkanFFT(vkGPU, &app, &launchParams, -1, num_iter);
 			if (resFFT != VKFFT_SUCCESS) return resFFT;
@@ -272,6 +305,23 @@ VkFFTResult sample_14_precision_VkFFT_single_nonPow2(VkGPU* vkGPU, uint64_t file
 #elif(VKFFT_BACKEND==3)
 				res = clEnqueueReadBuffer(vkGPU->commandQueue, buffer, CL_TRUE, 0, bufferSize[i], output_VkFFT, 0, NULL, NULL);
 				if (res != CL_SUCCESS) return VKFFT_ERROR_FAILED_TO_COPY;
+#elif(VKFFT_BACKEND==4)
+				ze_command_queue_desc_t commandQueueCopyDesc = {
+					ZE_STRUCTURE_TYPE_COMMAND_QUEUE_DESC,
+					0,
+					vkGPU->commandQueueID,
+					0, // index
+					0, // flags
+					ZE_COMMAND_QUEUE_MODE_DEFAULT,
+					ZE_COMMAND_QUEUE_PRIORITY_NORMAL
+				};
+				ze_command_list_handle_t copyCommandList;
+				res = zeCommandListCreateImmediate(vkGPU->context, vkGPU->device, &commandQueueCopyDesc, &copyCommandList);
+				if (res != ZE_RESULT_SUCCESS) return VKFFT_ERROR_FAILED_TO_CREATE_COMMAND_LIST;
+				res = zeCommandListAppendMemoryCopy(copyCommandList, output_VkFFT, buffer, bufferSize[i], 0, 0, 0);
+				if (res != ZE_RESULT_SUCCESS) return VKFFT_ERROR_FAILED_TO_COPY;
+				res = zeCommandQueueSynchronize(vkGPU->commandQueue, UINT32_MAX);
+				if (res != 0) return VKFFT_ERROR_FAILED_TO_SYNCHRONIZE;
 #endif
 				shift += bufferSize[i];
 			}
@@ -344,6 +394,8 @@ VkFFTResult sample_14_precision_VkFFT_single_nonPow2(VkGPU* vkGPU, uint64_t file
 				hipFree(buffer);
 #elif(VKFFT_BACKEND==3)
 				clReleaseMemObject(buffer);
+#elif(VKFFT_BACKEND==4)
+				zeMemFree(vkGPU->context, buffer);
 #endif
 
 			}
