@@ -37,6 +37,8 @@
 #else
 #include <CL/cl.h>
 #endif 
+#elif(VKFFT_BACKEND==4)
+#include <ze_api.h>
 #endif
 #include "vkFFT.h"
 #include "utils_VkFFT.h"
@@ -58,6 +60,8 @@ VkFFTResult sample_16_precision_VkFFT_single_dct(VkGPU* vkGPU, uint64_t file_out
 	hipError_t res = hipSuccess;
 #elif(VKFFT_BACKEND==3)
 	cl_int res = CL_SUCCESS;
+#elif(VKFFT_BACKEND==4)
+	ze_result_t res = ZE_RESULT_SUCCESS;
 #endif
 	if (file_output)
 		fprintf(output, "16 - VkFFT/FFTW R2R DCT-I, II, III and IV precision test in single precision\n");
@@ -162,8 +166,11 @@ VkFFTResult sample_16_precision_VkFFT_single_dct(VkGPU* vkGPU, uint64_t file_out
 				configuration.physicalDevice = &vkGPU->physicalDevice;
 				configuration.isCompilerInitialized = isCompilerInitialized;//compiler can be initialized before VkFFT plan creation. if not, VkFFT will create and destroy one after initialization
 #elif(VKFFT_BACKEND==3)
-				configuration.platform = &vkGPU->platform;
 				configuration.context = &vkGPU->context;
+#elif(VKFFT_BACKEND==4)
+				configuration.context = &vkGPU->context;
+				configuration.commandQueue = &vkGPU->commandQueue;
+				configuration.commandQueueID = vkGPU->commandQueueID;
 #endif			
 
 				uint64_t numBuf = 1;
@@ -186,6 +193,8 @@ VkFFTResult sample_16_precision_VkFFT_single_dct(VkGPU* vkGPU, uint64_t file_out
 				hipFloatComplex* buffer = 0;
 #elif(VKFFT_BACKEND==3)
 				cl_mem buffer = 0;
+#elif(VKFFT_BACKEND==4)
+				void* buffer = 0;
 #endif			
 				for (uint64_t i = 0; i < numBuf; i++) {
 #if(VKFFT_BACKEND==0)
@@ -202,6 +211,11 @@ VkFFTResult sample_16_precision_VkFFT_single_dct(VkGPU* vkGPU, uint64_t file_out
 #elif(VKFFT_BACKEND==3)
 					buffer = clCreateBuffer(vkGPU->context, CL_MEM_READ_WRITE, bufferSize[i], 0, &res);
 					if (res != CL_SUCCESS) return VKFFT_ERROR_FAILED_TO_ALLOCATE;
+#elif(VKFFT_BACKEND==4)
+					ze_device_mem_alloc_desc_t device_desc = {};
+					device_desc.stype = ZE_STRUCTURE_TYPE_DEVICE_MEM_ALLOC_DESC;
+					res = zeMemAllocDevice(vkGPU->context, &device_desc, bufferSize[i], sizeof(float), vkGPU->device, &buffer);
+					if (res != ZE_RESULT_SUCCESS) return VKFFT_ERROR_FAILED_TO_ALLOCATE;
 #endif
 				}
 
@@ -232,6 +246,23 @@ VkFFTResult sample_16_precision_VkFFT_single_dct(VkGPU* vkGPU, uint64_t file_out
 #elif(VKFFT_BACKEND==3)
 					res = clEnqueueWriteBuffer(vkGPU->commandQueue, buffer, CL_TRUE, 0, bufferSize[i], inputC, 0, NULL, NULL);
 					if (res != CL_SUCCESS) return VKFFT_ERROR_FAILED_TO_COPY;
+#elif(VKFFT_BACKEND==4)
+					ze_command_queue_desc_t commandQueueCopyDesc = {
+						ZE_STRUCTURE_TYPE_COMMAND_QUEUE_DESC,
+						0,
+						vkGPU->commandQueueID,
+						0, // index
+						0, // flags
+						ZE_COMMAND_QUEUE_MODE_DEFAULT,
+						ZE_COMMAND_QUEUE_PRIORITY_NORMAL
+					};
+					ze_command_list_handle_t copyCommandList;
+					res = zeCommandListCreateImmediate(vkGPU->context, vkGPU->device, &commandQueueCopyDesc, &copyCommandList);
+					if (res != ZE_RESULT_SUCCESS) return VKFFT_ERROR_FAILED_TO_CREATE_COMMAND_LIST;
+					res = zeCommandListAppendMemoryCopy(copyCommandList, buffer, inputC, bufferSize[i], 0, 0, 0);
+					if (res != ZE_RESULT_SUCCESS) return VKFFT_ERROR_FAILED_TO_COPY;
+					res = zeCommandQueueSynchronize(vkGPU->commandQueue, UINT32_MAX);
+					if (res != 0) return VKFFT_ERROR_FAILED_TO_SYNCHRONIZE;
 #endif
 					shift += bufferSize[i];
 				}
@@ -252,6 +283,8 @@ VkFFTResult sample_16_precision_VkFFT_single_dct(VkGPU* vkGPU, uint64_t file_out
 						hipFree(buffer);
 #elif(VKFFT_BACKEND==3)
 						clReleaseMemObject(buffer);
+#elif(VKFFT_BACKEND==4)
+						zeMemFree(vkGPU->context, buffer);
 #endif
 
 					}
@@ -280,6 +313,8 @@ VkFFTResult sample_16_precision_VkFFT_single_dct(VkGPU* vkGPU, uint64_t file_out
 				launchParams.buffer = (void**)&buffer;
 #elif(VKFFT_BACKEND==3)
 				launchParams.buffer = &buffer;
+#elif(VKFFT_BACKEND==4)
+				launchParams.buffer = (void**)&buffer;
 #endif
 				resFFT = performVulkanFFT(vkGPU, &app, &launchParams, -1, num_iter);
 				if (resFFT != VKFFT_SUCCESS) return resFFT;
@@ -300,6 +335,23 @@ VkFFTResult sample_16_precision_VkFFT_single_dct(VkGPU* vkGPU, uint64_t file_out
 #elif(VKFFT_BACKEND==3)
 					res = clEnqueueReadBuffer(vkGPU->commandQueue, buffer, CL_TRUE, 0, bufferSize[i], output_VkFFT, 0, NULL, NULL);
 					if (res != CL_SUCCESS) return VKFFT_ERROR_FAILED_TO_COPY;
+#elif(VKFFT_BACKEND==4)
+					ze_command_queue_desc_t commandQueueCopyDesc = {
+						ZE_STRUCTURE_TYPE_COMMAND_QUEUE_DESC,
+						0,
+						vkGPU->commandQueueID,
+						0, // index
+						0, // flags
+						ZE_COMMAND_QUEUE_MODE_DEFAULT,
+						ZE_COMMAND_QUEUE_PRIORITY_NORMAL
+					};
+					ze_command_list_handle_t copyCommandList;
+					res = zeCommandListCreateImmediate(vkGPU->context, vkGPU->device, &commandQueueCopyDesc, &copyCommandList);
+					if (res != ZE_RESULT_SUCCESS) return VKFFT_ERROR_FAILED_TO_CREATE_COMMAND_LIST;
+					res = zeCommandListAppendMemoryCopy(copyCommandList, output_VkFFT, buffer, bufferSize[i], 0, 0, 0);
+					if (res != ZE_RESULT_SUCCESS) return VKFFT_ERROR_FAILED_TO_COPY;
+					res = zeCommandQueueSynchronize(vkGPU->commandQueue, UINT32_MAX);
+					if (res != 0) return VKFFT_ERROR_FAILED_TO_SYNCHRONIZE;
 #endif
 					shift += bufferSize[i];
 				}
@@ -317,9 +369,9 @@ VkFFTResult sample_16_precision_VkFFT_single_dct(VkGPU* vkGPU, uint64_t file_out
 							//if (file_output) fprintf(output, "%.2e %.2e - %.2e %.2e \n", output_FFTW[i + j * dims[0] + l * dims[0] * dims[1]][0] / N, output_FFTW[i + j * dims[0] + l * dims[0] * dims[1]][1] / N, output_VkFFT[(loc_i + loc_j * dims[0] + loc_l * dims[0] * dims[1])][0], output_VkFFT[(loc_i + loc_j * dims[0] + loc_l * dims[0] * dims[1])][1]);
 
 							//printf("%.2e - %.2e \n", output_FFTW[i + j * dims[0] + l * dims[0] * dims[1]], output_VkFFT[(loc_i + loc_j * dims[0] + loc_l * dims[0] * dims[1])]);
-							double current_data_norm = sqrt(output_FFTW[i + j * dims[0] + l * dims[0] * dims[1]] * output_FFTW[i + j * dims[0] + l * dims[0] * dims[1]] + output_FFTW[i + j * dims[0] + l * dims[0] * dims[1]] * output_FFTW[i + j * dims[0] + l * dims[0] * dims[1]]);
+							double current_data_norm = sqrt(output_FFTW[i + j * dims[0] + l * dims[0] * dims[1]] * output_FFTW[i + j * dims[0] + l * dims[0] * dims[1]]);
 
-							double current_diff_norm_VkFFT = (output_VkFFT[loc_i + loc_j * dims[0] + loc_l * dims[0] * dims[1]] - output_FFTW[i + j * dims[0] + l * dims[0] * dims[1]]);
+							double current_diff_norm_VkFFT = sqrt((output_VkFFT[loc_i + loc_j * dims[0] + loc_l * dims[0] * dims[1]] - output_FFTW[i + j * dims[0] + l * dims[0] * dims[1]])*(output_VkFFT[loc_i + loc_j * dims[0] + loc_l * dims[0] * dims[1]] - output_FFTW[i + j * dims[0] + l * dims[0] * dims[1]]));
 							if (current_diff_norm_VkFFT > max_difference[1]) max_difference[1] = current_diff_norm_VkFFT;
 							avg_difference[1] += current_diff_norm_VkFFT;
 							if ((current_diff_norm_VkFFT / current_data_norm > max_eps[1])) {
@@ -348,6 +400,8 @@ VkFFTResult sample_16_precision_VkFFT_single_dct(VkGPU* vkGPU, uint64_t file_out
 					hipFree(buffer);
 #elif(VKFFT_BACKEND==3)
 					clReleaseMemObject(buffer);
+#elif(VKFFT_BACKEND==4)
+					zeMemFree(vkGPU->context, buffer);
 #endif
 
 				}
